##############################################################################
#
# Copyright (c) 2001-2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import unittest

from .common import BTreeTests
from .common import ExtendedSetTests
from .common import InternalKeysMappingTest
from .common import InternalKeysSetTest
from .common import MappingBase
from .common import MappingConflictTestBase
from .common import ModuleTest
from .common import MultiUnion
from .common import NormalSetTests
from .common import SetConflictTestBase
from .common import SetResult
from .common import TestLongIntKeys
from .common import makeBuilder


class LFBTreeInternalKeyTest(InternalKeysMappingTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFBTree
        return LFBTree


class LFBTreePyInternalKeyTest(InternalKeysMappingTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFBTreePy
        return LFBTreePy


class LFTreeSetInternalKeyTest(InternalKeysSetTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFTreeSet
        return LFTreeSet


class LFTreeSetPyInternalKeyTest(InternalKeysSetTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFTreeSetPy
        return LFTreeSetPy


class LFBucketTest(MappingBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFBucket
        return LFBucket


class LFBucketPyTest(MappingBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFBucketPy
        return LFBucketPy


class LFTreeSetTest(NormalSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFTreeSet
        return LFTreeSet


class LFTreeSetPyTest(NormalSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFTreeSetPy
        return LFTreeSetPy


class LFSetTest(ExtendedSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFSet
        return LFSet


class LFSetPyTest(ExtendedSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFSetPy
        return LFSetPy


class LFBTreeTest(BTreeTests, TestLongIntKeys, unittest.TestCase):

    def _makeOne(self):
        from BTrees.LFBTree import LFBTree
        return LFBTree()

    def getTwoValues(self):
        return 0.5, 1.5


class LFBTreePyTest(BTreeTests, TestLongIntKeys, unittest.TestCase):

    def _makeOne(self):
        from BTrees.LFBTree import LFBTreePy
        return LFBTreePy()

    def getTwoValues(self):
        return 0.5, 1.5


class TestLFMultiUnion(MultiUnion, unittest.TestCase):

    def multiunion(self, *args):
        from BTrees.LFBTree import multiunionPy
        return multiunionPy(*args)

    def union(self, *args):
        from BTrees.LFBTree import unionPy
        return unionPy(*args)

    def mkset(self, *args):
        from BTrees.LFBTree import LFSetPy as mkset
        return mkset(*args)

    def mktreeset(self, *args):
        from BTrees.LFBTree import LFTreeSetPy as mktreeset
        return mktreeset(*args)

    def mkbucket(self, *args):
        from BTrees.LFBTree import LFBucketPy as mkbucket
        return mkbucket(*args)

    def mkbtree(self, *args):
        from BTrees.LFBTree import LFBTreePy as mkbtree
        return mkbtree(*args)


class TestLFMultiUnionPy(MultiUnion, unittest.TestCase):

    def multiunion(self, *args):
        from BTrees.LFBTree import multiunionPy
        return multiunionPy(*args)

    def union(self, *args):
        from BTrees.LFBTree import unionPy
        return unionPy(*args)

    def mkset(self, *args):
        from BTrees.LFBTree import LFSetPy as mkset
        return mkset(*args)

    def mktreeset(self, *args):
        from BTrees.LFBTree import LFTreeSetPy as mktreeset
        return mktreeset(*args)

    def mkbucket(self, *args):
        from BTrees.LFBTree import LFBucketPy as mkbucket
        return mkbucket(*args)

    def mkbtree(self, *args):
        from BTrees.LFBTree import LFBTreePy as mkbtree
        return mkbtree(*args)


class PureLF(SetResult, unittest.TestCase):

    def union(self, *args):
        from BTrees.LFBTree import union
        return union(*args)

    def intersection(self, *args):
        from BTrees.LFBTree import intersection
        return intersection(*args)

    def difference(self, *args):
        from BTrees.LFBTree import difference
        return difference(*args)

    def builders(self):
        from BTrees.LFBTree import LFBTree
        from BTrees.LFBTree import LFBucket
        from BTrees.LFBTree import LFTreeSet
        from BTrees.LFBTree import LFSet
        return LFSet, LFTreeSet, makeBuilder(LFBTree), makeBuilder(LFBucket)


class PureLFPy(SetResult, unittest.TestCase):

    def union(self, *args):
        from BTrees.LFBTree import unionPy
        return unionPy(*args)

    def intersection(self, *args):
        from BTrees.LFBTree import intersectionPy
        return intersectionPy(*args)

    def difference(self, *args):
        from BTrees.LFBTree import differencePy
        return differencePy(*args)

    def builders(self):
        from BTrees.LFBTree import LFBTreePy
        from BTrees.LFBTree import LFBucketPy
        from BTrees.LFBTree import LFTreeSetPy
        from BTrees.LFBTree import LFSetPy
        return (LFSetPy, LFTreeSetPy,
                makeBuilder(LFBTreePy), makeBuilder(LFBucketPy))


class LFBTreeConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFBTree
        return LFBTree


class LFBTreeConflictTestsPy(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFBTreePy
        return LFBTreePy


class LFBucketConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFBucket
        return LFBucket


class LFBucketConflictTestsPy(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFBucketPy
        return LFBucketPy


class LFTreeSetConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFTreeSet
        return LFTreeSet


class LFTreeSetConflictTestsPy(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFTreeSetPy
        return LFTreeSetPy


class LFSetConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFSet
        return LFSet


class LFSetConflictTestsPy(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.LFBTree import LFSetPy
        return LFSetPy


class LFModuleTest(ModuleTest, unittest.TestCase):

    prefix = 'LF'

    def _getModule(self):
        import BTrees
        return BTrees.LFBTree

    def _getInterface(self):
        import BTrees.Interfaces
        return BTrees.Interfaces.IIntegerFloatBTreeModule


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(LFBTreeInternalKeyTest),
        unittest.makeSuite(LFBTreePyInternalKeyTest),
        unittest.makeSuite(LFTreeSetInternalKeyTest),
        unittest.makeSuite(LFTreeSetPyInternalKeyTest),
        unittest.makeSuite(LFBucketTest),
        unittest.makeSuite(LFBucketPyTest),
        unittest.makeSuite(LFTreeSetTest),
        unittest.makeSuite(LFTreeSetPyTest),
        unittest.makeSuite(LFSetTest),
        unittest.makeSuite(LFSetPyTest),
        unittest.makeSuite(LFBTreeTest),
        unittest.makeSuite(LFBTreePyTest),
        unittest.makeSuite(TestLFMultiUnion),
        unittest.makeSuite(TestLFMultiUnionPy),
        unittest.makeSuite(PureLF),
        unittest.makeSuite(PureLFPy),
        unittest.makeSuite(LFBTreeConflictTests),
        unittest.makeSuite(LFBTreeConflictTestsPy),
        unittest.makeSuite(LFBucketConflictTests),
        unittest.makeSuite(LFBucketConflictTestsPy),
        unittest.makeSuite(LFTreeSetConflictTests),
        unittest.makeSuite(LFTreeSetConflictTestsPy),
        unittest.makeSuite(LFSetConflictTests),
        unittest.makeSuite(LFSetConflictTestsPy),
        unittest.makeSuite(LFModuleTest),
    ))



##############################################################################
#
# Copyright (c) 2001-2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import os
import sys

PYPY = hasattr(sys, 'pypy_version_info')
# We can and do build the C extensions on PyPy, but
# as of Persistent 4.2.5 the persistent C extension is not
# built on PyPy, so importing our C extension will fail anyway.
PURE_PYTHON = os.environ.get('PURE_PYTHON', PYPY)


if sys.version_info[0] < 3: #pragma NO COVER Python2

    PY2 = True
    PY3 = False

    int_types = int, long
    xrange = xrange
    def compare(x, y):
        if x is None:
            if y is None:
                return 0
            else:
                return -1
        elif y is None:
            return 1
        else:
            return cmp(x, y)

    _bytes = str
    def _ascii(x):
        return bytes(x)

else: #pragma NO COVER Python3

    PY2 = False
    PY3 = True

    int_types = int,
    xrange = range

    def compare(x, y):
        if x is None:
            if y is None:
                return 0
            else:
                return -1
        elif y is None:
            return 1
        else:
            return (x > y) - (y > x)

    _bytes = bytes
    def _ascii(x):
        return bytes(x, 'ascii')


def import_c_extension(mod_globals):
    import importlib
    c_module = None
    module_name = mod_globals['__name__']
    assert module_name.startswith('BTrees.')
    module_name = module_name.split('.')[1]
    if not PURE_PYTHON:
        try:
            c_module = importlib.import_module('BTrees._' + module_name)
        except ImportError:
            pass
    if c_module is not None:
        new_values = dict(c_module.__dict__)
        new_values.pop("__name__", None)
        new_values.pop('__file__', None)
        new_values.pop('__doc__', None)
        mod_globals.update(new_values)
    else:
        # No C extension, make the Py versions available without that
        # extension. The list comprehension both filters and prevents
        # concurrent modification errors.
        for py in [k for k in mod_globals if k.endswith('Py')]:
            mod_globals[py[:-2]] = mod_globals[py]

    # Assign the global aliases
    prefix = module_name[:2]
    for name in ('Bucket', 'Set', 'BTree', 'TreeSet'):
        mod_globals[name] = mod_globals[prefix + name]

    # Cleanup
    del mod_globals['import_c_extension']



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import unittest


_marker = object()

class LengthTestCase(unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.Length import Length
        return Length

    def _makeOne(self, value=_marker):
        if value is _marker:
            return self._getTargetClass()()
        return self._getTargetClass()(value)

    def test_ctor_default(self):
        length = self._makeOne()
        self.assertEqual(length.value, 0)

    def test_ctor_explict(self):
        length = self._makeOne(42)
        self.assertEqual(length.value, 42)

    def test___getstate___(self):
        length = self._makeOne(42)
        self.assertEqual(length.__getstate__(), 42)

    def test___setstate__(self):
        length = self._makeOne()
        length.__setstate__(42)
        self.assertEqual(length.value, 42)

    def test_set(self):
        length = self._makeOne()
        length.set(42)
        self.assertEqual(length.value, 42)

    def test__p_resolveConflict(self):
        length = self._makeOne()
        self.assertEqual(length._p_resolveConflict(5, 7, 9), 11)

    def test_change_w_positive_delta(self):
        length = self._makeOne()
        length.change(3)
        self.assertEqual(length.value, 3)

    def test_change_w_negative_delta(self):
        length = self._makeOne()
        length.change(-3)
        self.assertEqual(length.value, -3)

    def test_change_overflows_to_long(self):
        import sys
        try:
            length = self._makeOne(sys.maxint)
        except AttributeError: #pragma NO COVER Py3k
            return
        else: #pragma NO COVER Py2
            self.assertEqual(length(), sys.maxint)
            self.assertTrue(type(length()) is int)
            length.change(+1)
            self.assertEqual(length(), sys.maxint + 1)
            self.assertTrue(type(length()) is long)

    def test_change_underflows_to_long(self):
        import sys
        try:
            minint = (-sys.maxint) - 1
        except AttributeError: #pragma NO COVER Py3k
            return
        else: #pragma NO COVER Py2
            length = self._makeOne(minint)
            self.assertEqual(length(), minint)
            self.assertTrue(type(length()) is int)
            length.change(-1)
            self.assertEqual(length(), minint - 1)
            self.assertTrue(type(length()) is long)

    def test___call___no_args(self):
        length = self._makeOne(42)
        self.assertEqual(length(), 42)

    def test___call___w_args(self):
        length = self._makeOne(42)
        self.assertEqual(length(0, None, (), [], {}), 42)

    def test_lp_516653(self):
        # Test for https://bugs.launchpad.net/zodb/+bug/516653
        import copy
        length = self._makeOne()
        other = copy.copy(length)
        self.assertEqual(other(), 0)


def test_suite():
    return unittest.makeSuite(LengthTestCase)



##############################################################################
#
# Copyright (c) 2001-2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

__all__ = ('Bucket', 'Set', 'BTree', 'TreeSet',
           'OIBucket', 'OISet', 'OIBTree', 'OITreeSet',
           'union', 'intersection', 'difference',
           'weightedUnion', 'weightedIntersection',
          )

from zope.interface import moduleProvides

from .Interfaces import IObjectIntegerBTreeModule
from ._base import Bucket
from ._base import MERGE
from ._base import MERGE_WEIGHT_numeric
from ._base import MERGE_DEFAULT_float
from ._base import Set
from ._base import Tree as BTree
from ._base import TreeSet
from ._base import _TreeIterator
from ._base import difference as _difference
from ._base import intersection as _intersection
from ._base import set_operation as _set_operation
from ._base import to_ob as _to_key
from ._base import to_int as _to_value
from ._base import union as _union
from ._base import weightedIntersection as _weightedIntersection
from ._base import weightedUnion as _weightedUnion
from ._base import _fix_pickle
from ._compat import import_c_extension

_BUCKET_SIZE = 60
_TREE_SIZE = 250
using64bits = True

class OIBucketPy(Bucket):
    _to_key = _to_key
    _to_value = _to_value
    MERGE = MERGE
    MERGE_WEIGHT = MERGE_WEIGHT_numeric
    MERGE_DEFAULT = MERGE_DEFAULT_float


class OISetPy(Set):
    _to_key = _to_key
    MERGE = MERGE
    MERGE_WEIGHT = MERGE_WEIGHT_numeric
    MERGE_DEFAULT = MERGE_DEFAULT_float


class OIBTreePy(BTree):
    max_leaf_size = _BUCKET_SIZE
    max_internal_size = _TREE_SIZE
    _to_key = _to_key
    _to_value = _to_value
    MERGE = MERGE
    MERGE_WEIGHT = MERGE_WEIGHT_numeric
    MERGE_DEFAULT = MERGE_DEFAULT_float


class OITreeSetPy(TreeSet):
    max_leaf_size = _BUCKET_SIZE
    max_internal_size = _TREE_SIZE
    _to_key = _to_key
    MERGE = MERGE
    MERGE_WEIGHT = MERGE_WEIGHT_numeric
    MERGE_DEFAULT = MERGE_DEFAULT_float


class OITreeIteratorPy(_TreeIterator):
    pass


# Can't declare forward refs, so fix up afterwards:

OIBucketPy._mapping_type = OIBucketPy._bucket_type = OIBucketPy
OIBucketPy._set_type = OISetPy

OISetPy._mapping_type = OIBucketPy
OISetPy._set_type = OISetPy._bucket_type = OISetPy

OIBTreePy._mapping_type = OIBTreePy._bucket_type = OIBucketPy
OIBTreePy._set_type = OISetPy

OITreeSetPy._mapping_type = OIBucketPy
OITreeSetPy._set_type = OITreeSetPy._bucket_type = OISetPy


differencePy = _set_operation(_difference, OISetPy)
unionPy = _set_operation(_union, OISetPy)
intersectionPy = _set_operation(_intersection, OISetPy)
weightedUnionPy = _set_operation(_weightedUnion, OISetPy)
weightedIntersectionPy = _set_operation(_weightedIntersection, OISetPy)

import_c_extension(globals())

_fix_pickle(globals(), __name__)

moduleProvides(IObjectIntegerBTreeModule)



##############################################################################
#
# Copyright (c) 2001-2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import unittest

from .common import BTreeTests
from .common import ExtendedSetTests
from .common import InternalKeysMappingTest
from .common import InternalKeysSetTest
from .common import MappingBase
from .common import MappingConflictTestBase
from .common import ModuleTest
from .common import MultiUnion
from .common import NormalSetTests
from .common import SetConflictTestBase
from .common import SetResult
from .common import TestLongIntKeys
from .common import makeBuilder
from BTrees.IIBTree import using64bits #XXX Ugly, but unavoidable


class IFBTreeInternalKeyTest(InternalKeysMappingTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFBTree
        return IFBTree


class IFBTreePyInternalKeyTest(InternalKeysMappingTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFBTreePy
        return IFBTreePy


class IFTreeSetInternalKeyTest(InternalKeysSetTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFTreeSet
        return IFTreeSet


class IFTreeSetPyInternalKeyTest(InternalKeysSetTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFTreeSetPy
        return IFTreeSetPy


class IFBucketTest(MappingBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFBucket
        return IFBucket


class IFBucketPyTest(MappingBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFBucketPy
        return IFBucketPy


class IFTreeSetTest(NormalSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFTreeSet
        return IFTreeSet


class IFTreeSetPyTest(NormalSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFTreeSetPy
        return IFTreeSetPy


class IFSetTest(ExtendedSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFSet
        return IFSet


class IFSetPyTest(ExtendedSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFSetPy
        return IFSetPy


class IFBTreeTest(BTreeTests, unittest.TestCase):

    def _makeOne(self):
        from BTrees.IFBTree import IFBTree
        return IFBTree()


class IFBTreePyTest(BTreeTests, unittest.TestCase):

    def _makeOne(self):
        from BTrees.IFBTree import IFBTreePy
        return IFBTreePy()

if using64bits:

    class IFBTreeTest(BTreeTests, TestLongIntKeys, unittest.TestCase):

        def _makeOne(self):
            from BTrees.IFBTree import IFBTree
            return IFBTree()

        def getTwoValues(self):
            return 0.5, 1.5

    class IFBTreePyTest(BTreeTests, TestLongIntKeys, unittest.TestCase):

        def _makeOne(self):
            from BTrees.IFBTree import IFBTreePy
            return IFBTreePy()

        def getTwoValues(self):
            return 0.5, 1.5


class _TestIFBTreesBase(object):

    def testNonIntegerKeyRaises(self):
        self.assertRaises(TypeError, self._stringraiseskey)
        self.assertRaises(TypeError, self._floatraiseskey)
        self.assertRaises(TypeError, self._noneraiseskey)

    def testNonNumericValueRaises(self):
        self.assertRaises(TypeError, self._stringraisesvalue)
        self.assertRaises(TypeError, self._noneraisesvalue)
        self._makeOne()[1] = 1
        self._makeOne()[1] = 1.0

    def _stringraiseskey(self):
        self._makeOne()['c'] = 1

    def _floatraiseskey(self):
        self._makeOne()[2.5] = 1

    def _noneraiseskey(self):
        self._makeOne()[None] = 1

    def _stringraisesvalue(self):
        self._makeOne()[1] = 'c'

    def _floatraisesvalue(self):
        self._makeOne()[1] = 1.4

    def _noneraisesvalue(self):
        self._makeOne()[1] = None


class TestIFBTrees(_TestIFBTreesBase, unittest.TestCase):

    def _makeOne(self):
        from BTrees.IFBTree import IFBTree
        return IFBTree()


class TestIFBTreesPy(_TestIFBTreesBase, unittest.TestCase):

    def _makeOne(self):
        from BTrees.IFBTree import IFBTreePy
        return IFBTreePy()


class TestIFMultiUnion(MultiUnion, unittest.TestCase):

    def multiunion(self, *args):
        from BTrees.IFBTree import multiunion
        return multiunion(*args)

    def union(self, *args):
        from BTrees.IFBTree import union
        return union(*args)

    def mkset(self, *args):
        from BTrees.IFBTree import IFSet as mkset
        return mkset(*args)

    def mktreeset(self, *args):
        from BTrees.IFBTree import IFTreeSet as mktreeset
        return mktreeset(*args)

    def mkbucket(self, *args):
        from BTrees.IFBTree import IFBucket as mkbucket
        return mkbucket(*args)

    def mkbtree(self, *args):
        from BTrees.IFBTree import IFBTree as mkbtree
        return mkbtree(*args)


class TestIFMultiUnionPy(MultiUnion, unittest.TestCase):

    def multiunion(self, *args):
        from BTrees.IFBTree import multiunionPy
        return multiunionPy(*args)

    def union(self, *args):
        from BTrees.IFBTree import unionPy
        return unionPy(*args)

    def mkset(self, *args):
        from BTrees.IFBTree import IFSetPy as mkset
        return mkset(*args)

    def mktreeset(self, *args):
        from BTrees.IFBTree import IFTreeSetPy as mktreeset
        return mktreeset(*args)

    def mkbucket(self, *args):
        from BTrees.IFBTree import IFBucketPy as mkbucket
        return mkbucket(*args)

    def mkbtree(self, *args):
        from BTrees.IFBTree import IFBTreePy as mkbtree
        return mkbtree(*args)


class PureIF(SetResult, unittest.TestCase):

    def union(self, *args):
        from BTrees.IFBTree import union
        return union(*args)

    def intersection(self, *args):
        from BTrees.IFBTree import intersection
        return intersection(*args)

    def difference(self, *args):
        from BTrees.IFBTree import difference
        return difference(*args)

    def builders(self):
        from BTrees.IFBTree import IFBTree
        from BTrees.IFBTree import IFBucket
        from BTrees.IFBTree import IFTreeSet
        from BTrees.IFBTree import IFSet
        return IFSet, IFTreeSet, makeBuilder(IFBTree), makeBuilder(IFBucket)


class PureIFPy(SetResult, unittest.TestCase):

    def union(self, *args):
        from BTrees.IFBTree import unionPy
        return unionPy(*args)

    def intersection(self, *args):
        from BTrees.IFBTree import intersectionPy
        return intersectionPy(*args)

    def difference(self, *args):
        from BTrees.IFBTree import differencePy
        return differencePy(*args)

    def builders(self):
        from BTrees.IFBTree import IFBTreePy
        from BTrees.IFBTree import IFBucketPy
        from BTrees.IFBTree import IFTreeSetPy
        from BTrees.IFBTree import IFSetPy
        return (IFSetPy, IFTreeSetPy,
                makeBuilder(IFBTreePy), makeBuilder(IFBucketPy))


class IFBTreeConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFBTree
        return IFBTree


class IFBTreePyConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFBTreePy
        return IFBTreePy


class IFBucketConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFBucket
        return IFBucket


class IFBucketPyConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFBucketPy
        return IFBucketPy


class IFTreeSetConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFTreeSet
        return IFTreeSet


class IFTreeSetPyConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFTreeSetPy
        return IFTreeSetPy


class IFSetConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFSet
        return IFSet


class IFSetPyConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.IFBTree import IFSetPy
        return IFSetPy


class IFModuleTest(ModuleTest, unittest.TestCase):

    prefix = 'IF'

    def _getModule(self):
        import BTrees
        return BTrees.IFBTree

    def _getInterface(self):
        import BTrees.Interfaces
        return BTrees.Interfaces.IIntegerFloatBTreeModule

def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(IFBTreeInternalKeyTest),
        unittest.makeSuite(IFBTreePyInternalKeyTest),
        unittest.makeSuite(IFTreeSetInternalKeyTest),
        unittest.makeSuite(IFTreeSetPyInternalKeyTest),
        unittest.makeSuite(IFBucketTest),
        unittest.makeSuite(IFBucketPyTest),
        unittest.makeSuite(IFTreeSetTest),
        unittest.makeSuite(IFTreeSetPyTest),
        unittest.makeSuite(IFSetTest),
        unittest.makeSuite(IFSetPyTest),
        unittest.makeSuite(IFBTreeTest),
        unittest.makeSuite(IFBTreePyTest),
        unittest.makeSuite(TestIFBTrees),
        unittest.makeSuite(TestIFBTreesPy),
        unittest.makeSuite(TestIFMultiUnion),
        unittest.makeSuite(TestIFMultiUnionPy),
        unittest.makeSuite(PureIF),
        unittest.makeSuite(PureIFPy),
        unittest.makeSuite(IFBTreeConflictTests),
        unittest.makeSuite(IFBTreePyConflictTests),
        unittest.makeSuite(IFBucketConflictTests),
        unittest.makeSuite(IFBucketPyConflictTests),
        unittest.makeSuite(IFTreeSetConflictTests),
        unittest.makeSuite(IFTreeSetPyConflictTests),
        unittest.makeSuite(IFSetConflictTests),
        unittest.makeSuite(IFSetPyConflictTests),
        unittest.makeSuite(IFModuleTest),
    ))



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import unittest

from BTrees.tests.common import permutations


class DegenerateBTree(unittest.TestCase):
    # Build a degenerate tree (set).  Boxes are BTree nodes.  There are
    # 5 leaf buckets, each containing a single int.  Keys in the BTree
    # nodes don't appear in the buckets.  Seven BTree nodes are purely
    # indirection nodes (no keys).  Buckets aren't all at the same depth:
    #
    #     +------------------------+
    #     |          4             |
    #     +------------------------+
    #         |              |
    #         |              v
    #         |             +-+
    #         |             | |
    #         |             +-+
    #         |              |
    #         v              v
    #     +-------+   +-------------+
    #     |   2   |   |   6     10  |
    #     +-------+   +-------------+
    #      |     |     |     |     |
    #      v     v     v     v     v
    #     +-+   +-+   +-+   +-+   +-+
    #     | |   | |   | |   | |   | |
    #     +-+   +-+   +-+   +-+   +-+
    #      |     |     |     |     |
    #      v     v     v     v     v
    #      1     3    +-+    7     11
    #                 | |
    #                 +-+
    #                  |
    #                  v
    #                  5
    #
    # This is nasty for many algorithms.  Consider a high-end range search
    # for 4.  The BTree nodes direct it to the 5 bucket, but the correct
    # answer is the 3 bucket, which requires going in a different direction
    # at the very top node already.  Consider a low-end range search for
    # 9.  The BTree nodes direct it to the 7 bucket, but the correct answer
    # is the 11 bucket.  This is also a nasty-case tree for deletions.

    def _build_degenerate_tree(self):
        # Build the buckets and chain them together.
        from BTrees.IIBTree import IISet
        from BTrees.IIBTree import IITreeSet
        from BTrees.check import check
        bucket11 = IISet([11])

        bucket7 = IISet()
        bucket7.__setstate__(((7,), bucket11))

        bucket5 = IISet()
        bucket5.__setstate__(((5,), bucket7))

        bucket3 = IISet()
        bucket3.__setstate__(((3,), bucket5))

        bucket1 = IISet()
        bucket1.__setstate__(((1,), bucket3))

        # Build the deepest layers of indirection nodes.
        ts = IITreeSet
        tree1 = ts()
        tree1.__setstate__(((bucket1,), bucket1))

        tree3 = ts()
        tree3.__setstate__(((bucket3,), bucket3))

        tree5lower = ts()
        tree5lower.__setstate__(((bucket5,), bucket5))
        tree5 = ts()
        tree5.__setstate__(((tree5lower,), bucket5))

        tree7 = ts()
        tree7.__setstate__(((bucket7,), bucket7))

        tree11 = ts()
        tree11.__setstate__(((bucket11,), bucket11))

        # Paste together the middle layers.
        tree13 = ts()
        tree13.__setstate__(((tree1, 2, tree3), bucket1))

        tree5711lower = ts()
        tree5711lower.__setstate__(((tree5, 6, tree7, 10, tree11), bucket5))
        tree5711 = ts()
        tree5711.__setstate__(((tree5711lower,), bucket5))

        # One more.
        t = ts()
        t.__setstate__(((tree13, 4, tree5711), bucket1))
        t._check()
        check(t)
        return t, [1, 3, 5, 7, 11]

    def testBasicOps(self):
        t, keys = self._build_degenerate_tree()
        self.assertEqual(len(t), len(keys))
        self.assertEqual(list(t.keys()), keys)
        # has_key actually returns the depth of a bucket.
        self.assertEqual(t.has_key(1), 4)
        self.assertEqual(t.has_key(3), 4)
        self.assertEqual(t.has_key(5), 6)
        self.assertEqual(t.has_key(7), 5)
        self.assertEqual(t.has_key(11), 5)
        for i in 0, 2, 4, 6, 8, 9, 10, 12:
            self.assertTrue(i not in t)

    def _checkRanges(self, tree, keys):
        self.assertEqual(len(tree), len(keys))
        sorted_keys = keys[:]
        sorted_keys.sort()
        self.assertEqual(list(tree.keys()), sorted_keys)
        for k in keys:
            self.assertTrue(k in tree)
        if keys:
            lokey = sorted_keys[0]
            hikey = sorted_keys[-1]
            self.assertEqual(lokey, tree.minKey())
            self.assertEqual(hikey, tree.maxKey())
        else:
            lokey = hikey = 42

        # Try all range searches.
        for lo in range(lokey - 1, hikey + 2):
            for hi in range(lo - 1, hikey + 2):
                for skipmin in False, True:
                    for skipmax in False, True:
                        wantlo, wanthi = lo, hi
                        if skipmin:
                            wantlo += 1
                        if skipmax:
                            wanthi -= 1
                        want = [k for k in keys if wantlo <= k <= wanthi]
                        got = list(tree.keys(lo, hi, skipmin, skipmax))
                        self.assertEqual(want, got)

    def testRanges(self):
        t, keys = self._build_degenerate_tree()
        self._checkRanges(t, keys)

    def testDeletes(self):
        # Delete keys in all possible orders, checking each tree along
        # the way.

        # This is a tough test.  Previous failure modes included:
        # 1. A variety of assertion failures in _checkRanges.
        # 2. Assorted "Invalid firstbucket pointer" failures at
        #    seemingly random times, coming out of the BTree destructor.
        # 3. Under Python 2.3 CVS, some baffling
        #      RuntimeWarning: tp_compare didn't return -1 or -2 for exception
        #    warnings, possibly due to memory corruption after a BTree
        #    goes insane.
        # On CPython in PURE_PYTHON mode, this is a *slow* test, taking 15+s
        # on a 2015 laptop.
        from BTrees.check import check
        t, keys = self._build_degenerate_tree()
        for oneperm in permutations(keys):
            t, keys = self._build_degenerate_tree()
            for key in oneperm:
                t.remove(key)
                keys.remove(key)
                t._check()
                check(t)
                self._checkRanges(t, keys)
            # We removed all the keys, so the tree should be empty now.
            self.assertEqual(t.__getstate__(), None)

            # A damaged tree may trigger an "invalid firstbucket pointer"
            # failure at the time its destructor is invoked.  Try to force
            # that to happen now, so it doesn't look like a baffling failure
            # at some unrelated line.
            del t   # trigger destructor


LP294788_ids = {}


class ToBeDeleted(object):
    def __init__(self, id):
        assert isinstance(id, int) #we don't want to store any object ref here
        self.id = id

        global LP294788_ids
        LP294788_ids[id] = 1

    def __del__(self):
        global LP294788_ids
        LP294788_ids.pop(self.id, None)

    def __cmp__(self, other):
        return cmp(self.id, other.id)

    def __le__(self, other):
        return self.id <= other.id

    def __lt__(self, other):
        return self.id < other.id

    def __eq__(self, other):
        return self.id == other.id

    def __ne__(self, other):
        return self.id != other.id

    def __gt__(self, other):
        return self.id > other.id

    def __ge__(self, other):
        return self.id >= other.id

    def __hash__(self):
        return hash(self.id)


class BugFixes(unittest.TestCase):

    # Collector 1843.  Error returns were effectively ignored in
    # Bucket_rangeSearch(), leading to "delayed" errors, or worse.
    def testFixed1843(self):
        from BTrees.IIBTree import IISet
        t = IISet()
        t.insert(1)
        # This one used to fail to raise the TypeError when it occurred.
        self.assertRaises(TypeError, t.keys, "")
        # This one used to segfault.
        self.assertRaises(TypeError, t.keys, 0, "")

    def test_LP294788(self):
        # https://bugs.launchpad.net/bugs/294788
        # BTree keeps some deleted objects referenced

        # The logic here together with the ToBeDeleted class is that
        # a separate reference dict is populated on object creation
        # and removed in __del__
        # That means what's left in the reference dict is never GC'ed
        # therefore referenced somewhere
        # To simulate real life, some random data is used to exercise the tree
        import gc
        import random
        from BTrees.OOBTree import OOBTree

        t = OOBTree()

        trandom = random.Random('OOBTree')

        global LP294788_ids

        # /// BTree keys are integers, value is an object
        LP294788_ids = {}
        ids = {}
        for i in range(1024):
            if trandom.random() > 0.1 or not ids:
                #add
                id = None
                while id is None or id in ids:
                    id = trandom.randint(0, 1000000)

                ids[id] = 1
                t[id] = ToBeDeleted(id)
            else:
                #del
                keys = list(ids.keys())
                if keys:
                    id = trandom.choice(list(ids.keys()))
                    del t[id]
                    del ids[id]

        ids = ids.keys()
        trandom.shuffle(list(ids))
        for id in ids:
            del t[id]
        ids = None

        #to be on the safe side run a full GC
        gc.collect()

        #print LP294788_ids

        self.assertEqual(len(t), 0)
        self.assertEqual(len(LP294788_ids), 0)
        # \\\

        # /// BTree keys are integers, value is a tuple having an object
        LP294788_ids = {}
        ids = {}
        for i in range(1024):
            if trandom.random() > 0.1 or not ids:
                #add
                id = None
                while id is None or id in ids:
                    id = trandom.randint(0, 1000000)

                ids[id] = 1
                t[id] = (id, ToBeDeleted(id), u'somename')
            else:
                #del
                keys = list(ids.keys())
                if keys:
                    id = trandom.choice(keys)
                    del t[id]
                    del ids[id]

        ids = ids.keys()
        trandom.shuffle(list(ids))
        for id in ids:
            del t[id]
        ids = None

        #to be on the safe side run a full GC
        gc.collect()

        #print LP294788_ids

        self.assertEqual(len(t), 0)
        self.assertEqual(len(LP294788_ids), 0)
        # \\\


        # /// BTree keys are objects, value is an int
        t = OOBTree()
        LP294788_ids = {}
        ids = {}
        for i in range(1024):
            if trandom.random() > 0.1 or not ids:
                #add
                id = None
                while id is None or id in ids:
                    id = ToBeDeleted(trandom.randint(0, 1000000))

                ids[id] = 1
                t[id] = 1
            else:
                #del
                id = trandom.choice(list(ids.keys()))
                del ids[id]
                del t[id]

        ids = ids.keys()
        trandom.shuffle(list(ids))
        for id in ids:
            del t[id]
        #release all refs
        ids = id = None

        #to be on the safe side run a full GC
        gc.collect()

        #print LP294788_ids

        self.assertEqual(len(t), 0)
        self.assertEqual(len(LP294788_ids), 0)

        # /// BTree keys are tuples having objects, value is an int
        t = OOBTree()
        LP294788_ids = {}
        ids = {}
        for i in range(1024):
            if trandom.random() > 0.1 or not ids:
                #add
                id = None
                while id is None or id in ids:
                    id = trandom.randint(0, 1000000)
                    id = (id, ToBeDeleted(id), u'somename')

                ids[id] = 1
                t[id] = 1
            else:
                #del
                id = trandom.choice(list(ids.keys()))
                del ids[id]
                del t[id]

        ids = ids.keys()
        trandom.shuffle(list(ids))
        for id in ids:
            del t[id]
        #release all refs
        ids = id = key = None

        #to be on the safe side run a full GC
        gc.collect()

        #print LP294788_ids

        self.assertEqual(len(t), 0)
        self.assertEqual(len(LP294788_ids), 0)


# cmp error propagation tests


class DoesntLikeBeingCompared:

    def __cmp__(self, other):
        raise ValueError('incomparable')
    __lt__ = __le__ = __eq__ = __ne__ = __ge__ = __gt__ = __cmp__

class TestCmpError(unittest.TestCase):

    def testFoo(self):
        from BTrees.OOBTree import OOBTree
        t = OOBTree()
        t['hello world'] = None
        try:
            t[DoesntLikeBeingCompared()] = None
        except ValueError as e:
            self.assertEqual(str(e), 'incomparable')
        else:
            self.fail('incomarable objects should not be allowed into '
                      'the tree')


class FamilyTest(unittest.TestCase):
    def test32(self):
        from zope.interface.verify import verifyObject
        import BTrees
        from BTrees.IOBTree import IOTreeSet
        verifyObject(BTrees.Interfaces.IBTreeFamily, BTrees.family32)
        self.assertEqual(
            BTrees.family32.IO, BTrees.IOBTree)
        self.assertEqual(
            BTrees.family32.OI, BTrees.OIBTree)
        self.assertEqual(
            BTrees.family32.II, BTrees.IIBTree)
        self.assertEqual(
            BTrees.family32.IF, BTrees.IFBTree)
        self.assertEqual(
            BTrees.family32.OO, BTrees.OOBTree)
        s = IOTreeSet()
        s.insert(BTrees.family32.maxint)
        self.assertTrue(BTrees.family32.maxint in s)
        s = IOTreeSet()
        s.insert(BTrees.family32.minint)
        self.assertTrue(BTrees.family32.minint in s)
        s = IOTreeSet()
        # this next bit illustrates an, um, "interesting feature".  If
        # the characteristics change to match the 64 bit version, please
        # feel free to change.
        with self.assertRaises((TypeError, OverflowError)):
            s.insert(BTrees.family32.maxint + 1)

        with self.assertRaises((TypeError, OverflowError)):
            s.insert(BTrees.family32.minint - 1)
        self.check_pickling(BTrees.family32)

    def test64(self):
        from zope.interface.verify import verifyObject
        import BTrees
        from BTrees.LOBTree import LOTreeSet
        verifyObject(BTrees.Interfaces.IBTreeFamily, BTrees.family64)
        self.assertEqual(
            BTrees.family64.IO, BTrees.LOBTree)
        self.assertEqual(
            BTrees.family64.OI, BTrees.OLBTree)
        self.assertEqual(
            BTrees.family64.II, BTrees.LLBTree)
        self.assertEqual(
            BTrees.family64.IF, BTrees.LFBTree)
        self.assertEqual(
            BTrees.family64.OO, BTrees.OOBTree)
        s = LOTreeSet()
        s.insert(BTrees.family64.maxint)
        self.assertTrue(BTrees.family64.maxint in s)
        s = LOTreeSet()
        s.insert(BTrees.family64.minint)
        self.assertTrue(BTrees.family64.minint in s)
        s = LOTreeSet()
        # XXX why oh why do we expect ValueError here, but TypeError in test32?
        self.assertRaises(ValueError, s.insert, BTrees.family64.maxint + 1)
        self.assertRaises(ValueError, s.insert, BTrees.family64.minint - 1)
        self.check_pickling(BTrees.family64)

    def check_pickling(self, family):
        # The "family" objects are singletons; they can be pickled and
        # unpickled, and the same instances will always be returned on
        # unpickling, whether from the same unpickler or different
        # unpicklers.
        import pickle
        from io import BytesIO

        s = pickle.dumps((family, family))
        (f1, f2) = pickle.loads(s)
        self.assertIs(f1, family)
        self.assertIs(f2, family)

        # Using a single memo across multiple pickles:
        sio = BytesIO()
        p = pickle.Pickler(sio)
        p.dump(family)
        p.dump([family])
        u = pickle.Unpickler(BytesIO(sio.getvalue()))
        f1 = u.load()
        f2, = u.load()
        self.assertTrue(f1 is family)
        self.assertTrue(f2 is family)

        # Using separate memos for each pickle:
        sio = BytesIO()
        p = pickle.Pickler(sio)
        p.dump(family)
        p.clear_memo()
        p.dump([family])
        u = pickle.Unpickler(BytesIO(sio.getvalue()))
        f1 = u.load()
        f2, = u.load()
        self.assertTrue(f1 is family)
        self.assertTrue(f2 is family)


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(DegenerateBTree),
        unittest.makeSuite(BugFixes),
        unittest.makeSuite(TestCmpError),
        unittest.makeSuite(FamilyTest),
    ))



##############################################################################
#
# Copyright (c) 2001-2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import unittest

from .common import _skip_under_Py3k
from .common import BTreeTests
from .common import ExtendedSetTests
from .common import InternalKeysMappingTest
from .common import InternalKeysSetTest
from .common import MappingBase
from .common import MappingConflictTestBase
from .common import ModuleTest
from .common import NormalSetTests
from .common import SetResult
from .common import SetConflictTestBase
from .common import makeBuilder



class OOBTreeInternalKeyTest(InternalKeysMappingTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOBTreePy
        return OOBTreePy

class OOBTreePyInternalKeyTest(InternalKeysMappingTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOBTree
        return OOBTree


class OOTreeSetInternalKeyTest(InternalKeysSetTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOTreeSet
        return OOTreeSet


class OOTreeSetPyInternalKeyTest(InternalKeysSetTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOTreeSetPy
        return OOTreeSetPy


class OOBucketTest(MappingBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOBucket
        return OOBucket


class OOBucketPyTest(MappingBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOBucketPy
        return OOBucketPy


class OOTreeSetTest(NormalSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOTreeSet
        return OOTreeSet


class OOTreeSetPyTest(NormalSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOTreeSetPy
        return OOTreeSetPy


class OOSetTest(ExtendedSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOSet
        return OOSet


class OOSetPyTest(ExtendedSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOSetPy
        return OOSetPy



class OOBTreeTest(BTreeTests, unittest.TestCase):

    def _makeOne(self, *args):
        from BTrees.OOBTree import OOBTree
        return OOBTree(*args)

    def test_byValue(self):
        ITEMS = [(y, x) for x, y in enumerate('abcdefghijklmnopqrstuvwxyz')]
        tree = self._makeOne(ITEMS)
        self.assertEqual(list(tree.byValue(22)),
                         [(y, x) for x, y in reversed(ITEMS[22:])])

    def testRejectDefaultComparisonOnSet(self):
        # Check that passing int keys w default comparison fails.
        # Only applies to new-style class instances. Old-style
        # instances are too hard to introspect.

        # This is white box because we know that the check is being
        # used in a function that's used in lots of places.
        # Otherwise, there are many permutations that would have to be
        # checked.
        from .._compat import PY2
        t = self._makeOne()

        class C(object):
            pass

        self.assertRaises(TypeError, lambda : t.__setitem__(C(), 1))

        with self.assertRaises(TypeError) as raising:
            t[C()] = 1

        self.assertEqual(raising.exception.args[0], "Object has default comparison")

        if PY2: # we only check for __cmp__ on Python2

            class With___cmp__(object):
                def __cmp__(*args):
                    return 1
            c = With___cmp__()
            t[c] = 1

            t.clear()

        class With___lt__(object):
            def __lt__(*args):
                return 1

        c = With___lt__()
        t[c] = 1

        t.clear()

    def testAcceptDefaultComparisonOnGet(self):
        # Issue #42
        t = self._makeOne()
        class C(object):
            pass

        self.assertEqual(t.get(C(), 42), 42)
        self.assertRaises(KeyError, t.__getitem__, C())
        self.assertFalse(C() in t)

    def test_None_is_smallest(self):
        t = self._makeOne()
        for i in range(999): # Make sure we multiple buckets
            t[i] = i*i
        t[None] = -1
        for i in range(-99,0): # Make sure we multiple buckets
            t[i] = i*i
        self.assertEqual(list(t), [None] + list(range(-99, 999)))
        self.assertEqual(list(t.values()),
                         [-1] + [i*i for i in range(-99, 999)])
        self.assertEqual(t[2], 4)
        self.assertEqual(t[-2], 4)
        self.assertEqual(t[None], -1)
        t[None] = -2
        self.assertEqual(t[None], -2)
        t2 = t.__class__(t)
        del t[None]
        self.assertEqual(list(t), list(range(-99, 999)))

        if 'Py' in self.__class__.__name__:
            return
        from BTrees.OOBTree import difference, union, intersection
        self.assertEqual(list(difference(t2, t).items()), [(None, -2)])
        self.assertEqual(list(union(t, t2)), list(t2))
        self.assertEqual(list(intersection(t, t2)), list(t))

    @_skip_under_Py3k
    def testDeleteNoneKey(self):
        # Check that a None key can be deleted in Python 2.
        # This doesn't work on Python 3 because None is unorderable,
        # so the tree can't be searched. But None also can't be inserted,
        # and we don't support migrating Python 2 databases to Python 3.
        t = self._makeOne()
        bucket_state = ((None, 42),)
        tree_state = ((bucket_state,),)
        t.__setstate__(tree_state)

        self.assertEqual(t[None], 42)
        del t[None]

    def testUnpickleNoneKey(self):
        # All versions (py2 and py3, C and Python) can unpickle
        # data that looks like this: {None: 42}, even though None
        # is unorderable..
        # This pickle was captured in BTree/ZODB3 3.10.7
        data = b'ccopy_reg\n__newobj__\np0\n(cBTrees.OOBTree\nOOBTree\np1\ntp2\nRp3\n((((NI42\ntp4\ntp5\ntp6\ntp7\nb.'

        import pickle
        t = pickle.loads(data)
        keys = list(t)
        self.assertEqual([None], keys)

    def testIdentityTrumpsBrokenComparison(self):
        # Identical keys always match, even if their comparison is
        # broken. See https://github.com/zopefoundation/BTrees/issues/50
        from functools import total_ordering

        @total_ordering
        class Bad(object):
            def __eq__(self, other):
                return False

            def __cmp__(self, other):
                return 1

            def __lt__(self, other):
                return False

        t = self._makeOne()
        bad_key = Bad()
        t[bad_key] = 42

        self.assertIn(bad_key, t)
        self.assertEqual(list(t), [bad_key])

        del t[bad_key]
        self.assertNotIn(bad_key, t)
        self.assertEqual(list(t), [])


class OOBTreePyTest(OOBTreeTest):
#
# Right now, we can't match the C extension's test / prohibition of the
# default 'object' comparison semantics.
#class OOBTreePyTest(BTreeTests, unittest.TestCase):

    def _makeOne(self, *args):
        from BTrees.OOBTree import OOBTreePy
        return OOBTreePy(*args)



class PureOO(SetResult, unittest.TestCase):

    def union(self, *args):
        from BTrees.OOBTree import union
        return union(*args)

    def intersection(self, *args):
        from BTrees.OOBTree import intersection
        return intersection(*args)

    def difference(self, *args):
        from BTrees.OOBTree import difference
        return difference(*args)

    def builders(self):
        from BTrees.OOBTree import OOBTree
        from BTrees.OOBTree import OOBucket
        from BTrees.OOBTree import OOTreeSet
        from BTrees.OOBTree import OOSet
        return OOSet, OOTreeSet, makeBuilder(OOBTree), makeBuilder(OOBucket)


class PureOOPy(SetResult, unittest.TestCase):

    def union(self, *args):
        from BTrees.OOBTree import unionPy
        return unionPy(*args)

    def intersection(self, *args):
        from BTrees.OOBTree import intersectionPy
        return intersectionPy(*args)

    def difference(self, *args):
        from BTrees.OOBTree import differencePy
        return differencePy(*args)

    def builders(self):
        from BTrees.OOBTree import OOBTreePy
        from BTrees.OOBTree import OOBucketPy
        from BTrees.OOBTree import OOTreeSetPy
        from BTrees.OOBTree import OOSetPy
        return (OOSetPy, OOTreeSetPy,
                makeBuilder(OOBTreePy), makeBuilder(OOBucketPy))


class OOBucketConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOBucket
        return OOBucket


class OOBucketPyConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOBucketPy
        return OOBucketPy


class OOSetConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOSet
        return OOSet


class OOSetPyConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOSetPy
        return OOSetPy


class OOBTreeConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOBTree
        return OOBTree


class OOBTreePyConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOBTreePy
        return OOBTreePy


class OOTreeSetConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOTreeSet
        return OOTreeSet


class OOTreeSetPyConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OOBTree import OOTreeSetPy
        return OOTreeSetPy


class OOModuleTest(ModuleTest, unittest.TestCase):

    prefix = 'OO'

    def _getModule(self):
        import BTrees
        return BTrees.OOBTree

    def _getInterface(self):
        import BTrees.Interfaces
        return BTrees.Interfaces.IObjectObjectBTreeModule

    def test_weightedUnion_not_present(self):
        try:
            from BTrees.OOBTree import weightedUnion
        except ImportError:
            pass
        else:
            self.fail("OOBTree shouldn't have weightedUnion")

    def test_weightedIntersection_not_present(self):
        try:
            from BTrees.OOBTree import weightedIntersection
        except ImportError:
            pass
        else:
            self.fail("OOBTree shouldn't have weightedIntersection")

    def test_multiunion_not_present(self):
        try:
            from BTrees.OOBTree import multiunion
        except ImportError:
            pass
        else:
            self.fail("OOBTree shouldn't have multiunion")


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(OOBTreeInternalKeyTest),
        unittest.makeSuite(OOBTreePyInternalKeyTest),
        unittest.makeSuite(OOTreeSetInternalKeyTest),
        unittest.makeSuite(OOTreeSetPyInternalKeyTest),
        unittest.makeSuite(OOBucketTest),
        unittest.makeSuite(OOBucketPyTest),
        unittest.makeSuite(OOTreeSetTest),
        unittest.makeSuite(OOTreeSetPyTest),
        unittest.makeSuite(OOSetTest),
        unittest.makeSuite(OOSetPyTest),
        unittest.makeSuite(OOBTreeTest),
        unittest.makeSuite(OOBTreePyTest),
        unittest.makeSuite(PureOO),
        unittest.makeSuite(PureOOPy),
        unittest.makeSuite(OOBucketConflictTests),
        unittest.makeSuite(OOBucketPyConflictTests),
        unittest.makeSuite(OOSetConflictTests),
        unittest.makeSuite(OOSetPyConflictTests),
        unittest.makeSuite(OOBTreeConflictTests),
        unittest.makeSuite(OOBTreePyConflictTests),
        unittest.makeSuite(OOTreeSetConflictTests),
        unittest.makeSuite(OOTreeSetPyConflictTests),
        unittest.makeSuite(OOModuleTest),
    ))



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import unittest


def _assertRaises(self, e_type, checked, *args, **kw):
    try:
        checked(*args, **kw)
    except e_type as e:
        return e
    self.fail("Didn't raise: %s" % e_type.__name__)


class Test_classify(unittest.TestCase):

    def _callFUT(self, obj):
        from BTrees.check import classify
        return classify(obj)

    def test_classify_w_unknown(self):
        class NotClassified(object):
            pass
        self.assertRaises(KeyError, self._callFUT, NotClassified())

    def test_classify_w_bucket(self):
        from BTrees.OOBTree import OOBucketPy
        from BTrees.check import TYPE_BUCKET
        kind, is_mapping = self._callFUT(OOBucketPy())
        self.assertEqual(kind, TYPE_BUCKET)
        self.assertTrue(is_mapping)

    def test_classify_w_set(self):
        from BTrees.OOBTree import OOSetPy
        from BTrees.check import TYPE_BUCKET
        kind, is_mapping = self._callFUT(OOSetPy())
        self.assertEqual(kind, TYPE_BUCKET)
        self.assertFalse(is_mapping)

    def test_classify_w_tree(self):
        from BTrees.OOBTree import OOBTreePy
        from BTrees.check import TYPE_BTREE
        kind, is_mapping = self._callFUT(OOBTreePy())
        self.assertEqual(kind, TYPE_BTREE)
        self.assertTrue(is_mapping)

    def test_classify_w_treeset(self):
        from BTrees.OOBTree import OOTreeSetPy
        from BTrees.check import TYPE_BTREE
        kind, is_mapping = self._callFUT(OOTreeSetPy())
        self.assertEqual(kind, TYPE_BTREE)
        self.assertFalse(is_mapping)


class Test_crack_btree(unittest.TestCase):

    def _callFUT(self, obj, is_mapping):
        from BTrees.check import crack_btree
        return crack_btree(obj, is_mapping)

    def test_w_empty_tree(self):
        from BTrees.check import BTREE_EMPTY
        class Empty(object):
            def __getstate__(self):
                return None
        kind, keys, kids = self._callFUT(Empty(), True)
        self.assertEqual(kind, BTREE_EMPTY)
        self.assertEqual(keys, [])
        self.assertEqual(kids, [])

    def test_w_degenerate_tree(self):
        from BTrees.check import BTREE_ONE
        class Degenerate(object):
            def __getstate__(self):
                return ((('a', 1, 'b', 2),),)
        kind, keys, kids = self._callFUT(Degenerate(), True)
        self.assertEqual(kind, BTREE_ONE)
        self.assertEqual(keys, ('a', 1, 'b', 2))
        self.assertEqual(kids, None)

    def test_w_normal_tree(self):
        from BTrees.check import BTREE_NORMAL
        first_bucket = [object()] * 8
        second_bucket = [object()] * 8
        class Normal(object):
            def __getstate__(self):
                return ((first_bucket, 'b', second_bucket), first_bucket)
        kind, keys, kids = self._callFUT(Normal(), True)
        self.assertEqual(kind, BTREE_NORMAL)
        self.assertEqual(keys, ['b'])
        self.assertEqual(kids, [first_bucket, second_bucket])


class Test_crack_bucket(unittest.TestCase):

    def _callFUT(self, obj, is_mapping):
        from BTrees.check import crack_bucket
        return crack_bucket(obj, is_mapping)

    def test_w_empty_set(self):
        class EmptySet(object):
            def __getstate__(self):
                return ([],)
        keys, values = self._callFUT(EmptySet(), False)
        self.assertEqual(keys, [])
        self.assertEqual(values, [])

    def test_w_non_empty_set(self):
        class NonEmptySet(object):
            def __getstate__(self):
                return (['a', 'b', 'c'],)
        keys, values = self._callFUT(NonEmptySet(), False)
        self.assertEqual(keys, ['a', 'b', 'c'])
        self.assertEqual(values, [])

    def test_w_empty_mapping(self):
        class EmptyMapping(object):
            def __getstate__(self):
                return ([], object())
        keys, values = self._callFUT(EmptyMapping(), True)
        self.assertEqual(keys, [])
        self.assertEqual(values, [])

    def test_w_non_empty_mapping(self):
        class NonEmptyMapping(object):
            def __getstate__(self):
                return (['a', 1, 'b', 2, 'c', 3], object())
        keys, values = self._callFUT(NonEmptyMapping(), True)
        self.assertEqual(keys, ['a', 'b', 'c'])
        self.assertEqual(values, [1, 2, 3])


class Test_type_and_adr(unittest.TestCase):

    def _callFUT(self, obj):
        from BTrees.check import type_and_adr
        return type_and_adr(obj)

    def test_type_and_adr_w_oid(self):
        from BTrees.utils import oid_repr
        class WithOid(object):
            _p_oid = b'DEADBEEF'
        t_and_a = self._callFUT(WithOid())
        self.assertTrue(t_and_a.startswith('WithOid (0x'))
        self.assertTrue(t_and_a.endswith('oid=%s)' % oid_repr(b'DEADBEEF')))

    def test_type_and_adr_wo_oid(self):
        class WithoutOid(object):
            pass
        t_and_a = self._callFUT(WithoutOid())
        self.assertTrue(t_and_a.startswith('WithoutOid (0x'))
        self.assertTrue(t_and_a.endswith('oid=None)'))


class WalkerTests(unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.check import Walker
        return Walker

    def _makeOne(self, obj):
        return self._getTargetClass()(obj)

    def test_visit_btree_abstract(self):
        walker = self._makeOne(object())
        obj = object()
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        self.assertRaises(NotImplementedError, walker.visit_btree,
                          obj, path, parent, is_mapping, keys, kids, lo, hi)

    def test_visit_bucket_abstract(self):
        walker = self._makeOne(object())
        obj = object()
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        self.assertRaises(NotImplementedError, walker.visit_bucket,
                          obj, path, parent, is_mapping, keys, kids, lo, hi)

    def test_walk_w_empty_bucket(self):
        from BTrees.OOBTree import OOBucket
        obj = OOBucket()
        walker = self._makeOne(obj)
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        self.assertRaises(NotImplementedError, walker.walk)

    def test_walk_w_empty_btree(self):
        from BTrees.OOBTree import OOBTree
        obj = OOBTree()
        walker = self._makeOne(obj)
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        self.assertRaises(NotImplementedError, walker.walk)

    def test_walk_w_degenerate_btree(self):
        from BTrees.OOBTree import OOBTree
        obj = OOBTree()
        obj['a'] = 1
        walker = self._makeOne(obj)
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        self.assertRaises(NotImplementedError, walker.walk)

    def test_walk_w_normal_btree(self):
        from BTrees.IIBTree import IIBTree
        obj = IIBTree()
        for i in range(1000):
            obj[i] = i
        walker = self._makeOne(obj)
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        self.assertRaises(NotImplementedError, walker.walk)


class CheckerTests(unittest.TestCase):

    assertRaises = _assertRaises

    def _getTargetClass(self):
        from BTrees.check import Checker
        return Checker

    def _makeOne(self, obj):
        return self._getTargetClass()(obj)

    def test_walk_w_empty_bucket(self):
        from BTrees.OOBTree import OOBucket
        obj = OOBucket()
        checker = self._makeOne(obj)
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        checker.check() #noraise

    def test_walk_w_empty_btree(self):
        obj = _makeTree(False)
        checker = self._makeOne(obj)
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        checker.check() #noraise

    def test_walk_w_degenerate_btree(self):
        obj = _makeTree(False)
        obj['a'] = 1
        checker = self._makeOne(obj)
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        checker.check() #noraise

    def test_walk_w_normal_btree(self):
        obj = _makeTree(False)
        checker = self._makeOne(obj)
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        checker.check() #noraise

    def test_walk_w_key_too_large(self):
        obj = _makeTree(True)
        state = obj.__getstate__()
        # Damage an invariant by dropping the BTree key to 14.
        new_state = (state[0][0], 14, state[0][2]), state[1]
        obj.__setstate__(new_state)
        checker = self._makeOne(obj)
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        e = self.assertRaises(AssertionError, checker.check)
        self.assertTrue(">= upper bound" in str(e))

    def test_walk_w_key_too_small(self):
        obj = _makeTree(True)
        state = obj.__getstate__()
        # Damage an invariant by bumping the BTree key to 16.
        new_state = (state[0][0], 16, state[0][2]), state[1]
        obj.__setstate__(new_state)
        checker = self._makeOne(obj)
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        e = self.assertRaises(AssertionError, checker.check)
        self.assertTrue("< lower bound" in str(e))

    def test_walk_w_keys_swapped(self):
        obj = _makeTree(True)
        state = obj.__getstate__()
        # Damage an invariant by bumping the BTree key to 16.
        (b0, num, b1), firstbucket = state
        self.assertEqual(b0[4], 8)
        self.assertEqual(b0[5], 10)
        b0state = b0.__getstate__()
        self.assertEqual(len(b0state), 2)
        # b0state looks like
        # ((k0, v0, k1, v1, ...), nextbucket)
        pairs, nextbucket = b0state
        self.assertEqual(pairs[8], 4)
        self.assertEqual(pairs[9], 8)
        self.assertEqual(pairs[10], 5)
        self.assertEqual(pairs[11], 10)
        newpairs = pairs[:8] + (5, 10, 4, 8) + pairs[12:]
        b0.__setstate__((newpairs, nextbucket))
        checker = self._makeOne(obj)
        path = '/'
        parent = object()
        is_mapping = True
        keys = []
        kids = []
        lo = 0
        hi = None
        e = self.assertRaises(AssertionError, checker.check)
        self.assertTrue("key 5 at index 4 >= key 4 at index 5" in str(e))


class Test_check(unittest.TestCase):

    def _callFUT(self, tree):
        from BTrees.check import check
        return check(tree)

    def _makeOne(self):
        from BTrees.OOBTree import OOBTree
        tree = OOBTree()
        for i in range(31):
            tree[i] = 2*i
        return tree

    def test_normal(self):
        from BTrees.OOBTree import OOBTree
        tree = OOBTree()
        for i in range(31):
            tree[i] = 2*i
        state = tree.__getstate__()
        self.assertEqual(len(state), 2)
        self.assertEqual(len(state[0]), 3)
        self.assertEqual(state[0][1], 15)
        self._callFUT(tree)   #noraise


def _makeTree(fill):
    from BTrees.OOBTree import OOBTree
    from BTrees.OOBTree import _BUCKET_SIZE
    tree = OOBTree()
    if fill:
        for i in range(_BUCKET_SIZE + 1):
            tree[i] = 2*i
    return tree


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(Test_classify),
        unittest.makeSuite(Test_crack_btree),
        unittest.makeSuite(Test_crack_bucket),
        unittest.makeSuite(Test_type_and_adr),
        unittest.makeSuite(WalkerTests),
        unittest.makeSuite(CheckerTests),
        unittest.makeSuite(Test_check),
    ))



##############################################################################
#
# Copyright (c) 2001-2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import unittest

from .common import BTreeTests
from .common import ExtendedSetTests
from .common import InternalKeysMappingTest
from .common import InternalKeysSetTest
from .common import MappingBase
from .common import MappingConflictTestBase
from .common import ModuleTest
from .common import NormalSetTests
from .common import SetConflictTestBase
from .common import SetResult
from .common import TestLongIntValues
from .common import TypeTest
from .common import Weighted
from .common import itemsToSet
from .common import makeBuilder
from BTrees.IIBTree import using64bits #XXX Ugly, but necessary


class OIBTreeInternalKeyTest(InternalKeysMappingTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OIBTree
        return OIBTree


class OIBTreePyInternalKeyTest(InternalKeysMappingTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OIBTreePy
        return OIBTreePy


class OITreeSetInternalKeyTest(InternalKeysSetTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OITreeSet
        return OITreeSet


class OITreeSetPyInternalKeyTest(InternalKeysSetTest, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OITreeSetPy
        return OITreeSetPy


class OIBucketTest(MappingBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OIBucket
        return OIBucket


class OIBucketPyTest(MappingBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OIBucketPy
        return OIBucketPy


class OITreeSetTest(NormalSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OITreeSet
        return OITreeSet


class OITreeSetPyTest(NormalSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OITreeSetPy
        return OITreeSetPy


class OISetTest(ExtendedSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OISet
        return OISet


class OISetPyTest(ExtendedSetTests, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OISetPy
        return OISetPy


class OIBTreeTest(BTreeTests, unittest.TestCase):

    def _makeOne(self):
        from BTrees.OIBTree import OIBTree
        return OIBTree()


class OIBTreePyTest(BTreeTests, unittest.TestCase):
    def _makeOne(self):
        from BTrees.OIBTree import OIBTreePy
        return OIBTreePy()


if using64bits:

    class OIBTreeTest(BTreeTests, TestLongIntValues, unittest.TestCase):
        def _makeOne(self):
            from BTrees.OIBTree import OIBTree
            return OIBTree()
        def getTwoKeys(self):
            return object(), object()

    class OIBTreePyTest(BTreeTests, TestLongIntValues, unittest.TestCase):
        def _makeOne(self):
            from BTrees.OIBTree import OIBTreePy
            return OIBTreePy()
        def getTwoKeys(self):
            return object(), object()


class _TestOIBTreesBase(TypeTest):

    def _stringraises(self):
        self._makeOne()[1] = 'c'

    def _floatraises(self):
        self._makeOne()[1] = 1.4

    def _noneraises(self):
        self._makeOne()[1] = None

    def testEmptyFirstBucketReportedByGuido(self):
        from .._compat import xrange
        b = self._makeOne()
        for i in xrange(29972): # reduce to 29971 and it works
            b[i] = i
        for i in xrange(30): # reduce to 29 and it works
            del b[i]
            b[i+40000] = i

        self.assertEqual(b.keys()[0], 30)


class TestOIBTrees(_TestOIBTreesBase, unittest.TestCase):

    def _makeOne(self):
        from BTrees.OIBTree import OIBTree
        return OIBTree()


class TestOIBTreesPy(_TestOIBTreesBase, unittest.TestCase):

    def _makeOne(self):
        from BTrees.OIBTree import OIBTreePy
        return OIBTreePy()


class PureOI(SetResult, unittest.TestCase):

    def union(self, *args):
        from BTrees.OIBTree import union
        return union(*args)

    def intersection(self, *args):
        from BTrees.OIBTree import intersection
        return intersection(*args)

    def difference(self, *args):
        from BTrees.OIBTree import difference
        return difference(*args)

    def builders(self):
        from BTrees.OIBTree import OIBTree
        from BTrees.OIBTree import OIBucket
        from BTrees.OIBTree import OITreeSet
        from BTrees.OIBTree import OISet
        return OISet, OITreeSet, makeBuilder(OIBTree), makeBuilder(OIBucket)


class PureOIPy(SetResult, unittest.TestCase):

    def union(self, *args):
        from BTrees.OIBTree import unionPy
        return unionPy(*args)

    def intersection(self, *args):
        from BTrees.OIBTree import intersectionPy
        return intersectionPy(*args)

    def difference(self, *args):
        from BTrees.OIBTree import differencePy
        return differencePy(*args)

    def builders(self):
        from BTrees.OIBTree import OIBTreePy
        from BTrees.OIBTree import OIBucketPy
        from BTrees.OIBTree import OITreeSetPy
        from BTrees.OIBTree import OISetPy
        return (OISetPy, OITreeSetPy,
                makeBuilder(OIBTreePy), makeBuilder(OIBucketPy))


class TestWeightedOI(Weighted, unittest.TestCase):

    def weightedUnion(self):
        from BTrees.OIBTree import weightedUnion
        return weightedUnion

    def weightedIntersection(self):
        from BTrees.OIBTree import weightedIntersection
        return weightedIntersection

    def union(self):
        from BTrees.OIBTree import union
        return union

    def intersection(self):
        from BTrees.OIBTree import intersection
        return intersection

    def mkbucket(self, *args):
        from BTrees.OIBTree import OIBucket as mkbucket
        return mkbucket(*args)

    def builders(self):
        from BTrees.OIBTree import OIBTree
        from BTrees.OIBTree import OIBucket
        from BTrees.OIBTree import OITreeSet
        from BTrees.OIBTree import OISet
        return OIBucket, OIBTree, itemsToSet(OISet), itemsToSet(OITreeSet)


class TestWeightedOIPy(Weighted, unittest.TestCase):

    def weightedUnion(self):
        from BTrees.OIBTree import weightedUnionPy
        return weightedUnionPy

    def weightedIntersection(self):
        from BTrees.OIBTree import weightedIntersectionPy
        return weightedIntersectionPy

    def union(self):
        from BTrees.OIBTree import unionPy
        return unionPy

    def intersection(self):
        from BTrees.OIBTree import intersectionPy
        return intersectionPy

    def mkbucket(self, *args):
        from BTrees.OIBTree import OIBucketPy as mkbucket
        return mkbucket(*args)

    def builders(self):
        from BTrees.OIBTree import OIBTreePy
        from BTrees.OIBTree import OIBucketPy
        from BTrees.OIBTree import OITreeSetPy
        from BTrees.OIBTree import OISetPy
        return (OIBucketPy, OIBTreePy,
                itemsToSet(OISetPy), itemsToSet(OITreeSetPy))


class OIBucketConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OIBucket
        return OIBucket


class OIBucketConflictTestsPy(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OIBucketPy
        return OIBucketPy


class OISetConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OISet
        return OISet


class OISetConflictTestsPy(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OISetPy
        return OISetPy


class OIBTreeConflictTests(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OIBTree
        return OIBTree


class OIBTreeConflictTestsPy(MappingConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OIBTreePy
        return OIBTreePy


class OITreeSetConflictTests(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OITreeSet
        return OITreeSet


class OITreeSetConflictTestsPy(SetConflictTestBase, unittest.TestCase):

    def _getTargetClass(self):
        from BTrees.OIBTree import OITreeSetPy
        return OITreeSetPy


class OIModuleTest(ModuleTest, unittest.TestCase):

    prefix = 'OI'

    def _getModule(self):
        import BTrees
        return BTrees.OIBTree

    def _getInterface(self):
        import BTrees.Interfaces
        return BTrees.Interfaces.IObjectIntegerBTreeModule

    def test_multiunion_not_present(self):
        try:
            from BTrees.OIBTree import multiunion
        except ImportError:
            pass
        else:
            self.fail("OIBTree shouldn't have multiunion")


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(OIBTreeInternalKeyTest),
        unittest.makeSuite(OIBTreePyInternalKeyTest),
        unittest.makeSuite(OITreeSetInternalKeyTest),
        unittest.makeSuite(OITreeSetPyInternalKeyTest),
        unittest.makeSuite(OIBucketTest),
        unittest.makeSuite(OIBucketPyTest),
        unittest.makeSuite(OITreeSetTest),
        unittest.makeSuite(OITreeSetPyTest),
        unittest.makeSuite(OISetTest),
        unittest.makeSuite(OISetPyTest),
        unittest.makeSuite(OIBTreeTest),
        unittest.makeSuite(OIBTreePyTest),
        unittest.makeSuite(TestOIBTrees),
        unittest.makeSuite(TestOIBTreesPy),
        unittest.makeSuite(PureOI),
        unittest.makeSuite(PureOIPy),
        unittest.makeSuite(TestWeightedOI),
        unittest.makeSuite(TestWeightedOIPy),
        unittest.makeSuite(OIBucketConflictTests),
        unittest.makeSuite(OIBucketConflictTestsPy),
        unittest.makeSuite(OISetConflictTests),
        unittest.makeSuite(OISetConflictTestsPy),
        unittest.makeSuite(OIBTreeConflictTests),
        unittest.makeSuite(OIBTreeConflictTestsPy),
        unittest.makeSuite(OITreeSetConflictTests),
        unittest.makeSuite(OITreeSetConflictTestsPy),
        unittest.makeSuite(OIModuleTest),
    ))



##############################################################################
#
# Copyright (c) 2001-2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

__all__ = ('Bucket', 'Set', 'BTree', 'TreeSet',
           'IIBucket', 'IISet', 'IIBTree', 'IITreeSet',
           'union', 'intersection', 'difference',
           'weightedUnion', 'weightedIntersection', 'multiunion',
          )

from zope.interface import moduleProvides

from .Interfaces import IIntegerIntegerBTreeModule
from ._base import Bucket
from ._base import MERGE
from ._base import MERGE_WEIGHT_numeric
from ._base import MERGE_DEFAULT_int
from ._base import Set
from ._base import Tree as BTree
from ._base import TreeSet
from ._base import _TreeIterator
from ._base import difference as _difference
from ._base import intersection as _intersection
from ._base import multiunion as _multiunion
from ._base import set_operation as _set_operation
from ._base import to_int as _to_key
_to_value = _to_key
from ._base import union as _union
from ._base import weightedIntersection as _weightedIntersection
from ._base import weightedUnion as _weightedUnion
from ._base import _fix_pickle
from ._compat import import_c_extension

_BUCKET_SIZE = 120
_TREE_SIZE = 500
using64bits = False


class IIBucketPy(Bucket):
    _to_key = _to_key
    _to_value = _to_value
    MERGE = MERGE
    MERGE_WEIGHT = MERGE_WEIGHT_numeric
    MERGE_DEFAULT = MERGE_DEFAULT_int


class IISetPy(Set):
    _to_key = _to_key
    MERGE = MERGE
    MERGE_WEIGHT = MERGE_WEIGHT_numeric
    MERGE_DEFAULT = MERGE_DEFAULT_int


class IIBTreePy(BTree):
    max_leaf_size = _BUCKET_SIZE
    max_internal_size = _TREE_SIZE
    _to_key = _to_key
    _to_value = _to_value
    MERGE = MERGE
    MERGE_WEIGHT = MERGE_WEIGHT_numeric
    MERGE_DEFAULT = MERGE_DEFAULT_int


class IITreeSetPy(TreeSet):
    max_leaf_size = _BUCKET_SIZE
    max_internal_size = _TREE_SIZE
    _to_key = _to_key
    MERGE = MERGE
    MERGE_WEIGHT = MERGE_WEIGHT_numeric
    MERGE_DEFAULT = MERGE_DEFAULT_int


class IITreeIteratorPy(_TreeIterator):
    pass


# Can't declare forward refs, so fix up afterwards:

IIBucketPy._mapping_type = IIBucketPy._bucket_type = IIBucketPy
IIBucketPy._set_type = IISetPy

IISetPy._mapping_type = IIBucketPy
IISetPy._set_type = IISetPy._bucket_type = IISetPy

IIBTreePy._mapping_type = IIBTreePy._bucket_type = IIBucketPy
IIBTreePy._set_type = IISetPy

IITreeSetPy._mapping_type = IIBucketPy
IITreeSetPy._set_type = IITreeSetPy._bucket_type = IISetPy


differencePy = _set_operation(_difference, IISetPy)
unionPy = _set_operation(_union, IISetPy)
intersectionPy = _set_operation(_intersection, IISetPy)
multiunionPy = _set_operation(_multiunion, IISetPy)
weightedUnionPy = _set_operation(_weightedUnion, IISetPy)
weightedIntersectionPy = _set_operation(_weightedIntersection, IISetPy)

import_c_extension(globals())

_fix_pickle(globals(), __name__)

moduleProvides(IIntegerIntegerBTreeModule)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import unittest
from .common import _skip_under_Py3k

# When an OOBtree contains unicode strings as keys,
# it is neccessary accessing non-unicode strings are
# either ascii strings or encoded as unicoded using the
# corresponding encoding

encoding = 'ISO-8859-1'

class TestBTreesUnicode(unittest.TestCase):
    """ test unicode"""

    def setUp(self):
        #setup an OOBTree with some unicode strings
        from BTrees.OOBTree import OOBTree
        from BTrees._compat import _bytes

        self.s = b'dreit\xe4gigen'.decode('latin1')

        self.data = [(b'alien', 1),
                     (b'k\xf6nnten', 2),
                     (b'fox', 3),
                     (b'future', 4),
                     (b'quick', 5),
                     (b'zerst\xf6rt', 6),
                     (u'dreit\xe4gigen', 7),
                    ]

        self.tree = OOBTree()
        for k, v in self.data:
            if isinstance(k, _bytes):
                k = k.decode('latin1')
            self.tree[k] = v

    @_skip_under_Py3k
    def testAllKeys(self):
        # check every item of the tree
        from BTrees._compat import _bytes
        for k, v in self.data:
            if isinstance(k, _bytes):
                k = k.decode(encoding)
            self.assertTrue(k in self.tree)
            self.assertEqual(self.tree[k], v)

    @_skip_under_Py3k
    def testUnicodeKeys(self):
        # try to access unicode keys in tree
        k, v = self.data[-1]
        self.assertEqual(k, self.s)
        self.assertEqual(self.tree[k], v)
        self.assertEqual(self.tree[self.s], v)

    @_skip_under_Py3k
    def testAsciiKeys(self):
        # try to access some "plain ASCII" keys in the tree
        for k, v in self.data[0], self.data[2]:
            self.assertTrue(isinstance(k, str))
            self.assertEqual(self.tree[k], v)

def test_suite():
    return unittest.makeSuite(TestBTreesUnicode)
